@@ -63,6 +63,7 @@ SELECT oracle_execute(
           '   vc  VARCHAR2(10 CHAR),\n'
           '   nvc NVARCHAR2(10),\n'
           '   lc  CLOB,\n'
+          '   lnc NCLOB,\n'
           '   r   RAW(10),\n'
           '   u   RAW(16),\n'
           '   lb  BLOB,\n'
@@ -152,6 +153,7 @@ CREATE FOREIGN TABLE typetest1 (
    vc  character varying(10),
    nvc character varying(10),
    lc  text,
+   lnc text,
    r   bytea,
    u   uuid,
    lb  bytea,
@@ -181,6 +183,7 @@ CREATE FOREIGN TABLE longy (
    vc  character varying(10),
    nvc character varying(10),
    lc  text,
+   lnc text,
    r   bytea,
    u   uuid,
    lb  bytea,
@@ -217,13 +220,14 @@ ALTER SERVER oracle OPTIONS (DROP isolation_level);
 SELECT oracle_close_connections();
 DELETE FROM typetest1;
 
-INSERT INTO typetest1 (id, c, nc, vc, nvc, lc, r, u, lb, lr, b, num, fl, db, d, ts, ids, iym) VALUES (
+INSERT INTO typetest1 (id, c, nc, vc, nvc, lc, lnc, r, u, lb, lr, b, num, fl, db, d, ts, ids, iym) VALUES (
    1,
    'fixed char',
    'nat''l char',
    'varlena',
    'nat''l var',
    'character large object',
+   'character national large object',
    bytea('\xDEADBEEF'),
    uuid('055e26fa-f1d8-771f-e053-1645990add93'),
    bytea('\xDEADBEEF'),
@@ -243,13 +247,14 @@ SELECT oracle_execute('oracle', 'UPDATE typetest1 SET b = 2 WHERE id = 1');
 
 INSERT INTO shorty (id, c) VALUES (2, NULL);
 
-INSERT INTO typetest1 (id, c, nc, vc, nvc, lc, r, u, lb, lr, b, num, fl, db, d, ts, ids, iym) VALUES (
+INSERT INTO typetest1 (id, c, nc, vc, nvc, lc, lnc, r, u, lb, lr, b, num, fl, db, d, ts, ids, iym) VALUES (
    3,
    E'a\u001B\u0007\u000D\u007Fb',
    E'a\u001B\u0007\u000D\u007Fb',
    E'a\u001B\u0007\u000D\u007Fb',
    E'a\u001B\u0007\u000D\u007Fb',
    E'a\u001B\u0007\u000D\u007Fb ABC' || repeat('X', 9000),
+   E'a\u001B\u0007\u000D\u007Fb ABC' || repeat('X', 9000),
    bytea('\xDEADF00D'),
    uuid('055f3b32-a02c-4532-e053-1645990a6db2'),
    bytea('\xDEADF00DDEADF00DDEADF00D'),
@@ -264,13 +269,14 @@ INSERT INTO typetest1 (id, c, nc, vc, nvc, lc, r, u, lb, lr, b, num, fl, db, d,
    '-2 years -6 months'
 );
 
-INSERT INTO typetest1 (id, c, nc, vc, nvc, lc, r, u, lb, lr, b, num, fl, db, d, ts, ids, iym) VALUES (
+INSERT INTO typetest1 (id, c, nc, vc, nvc, lc, lnc, r, u, lb, lr, b, num, fl, db, d, ts, ids, iym) VALUES (
    4,
    'short',
    'short',
    'short',
    'short',
    'short',
+   'short',
    bytea('\xDEADF00D'),
    uuid('0560ee34-2ef9-1137-e053-1645990ac874'),
    bytea('\xDEADF00D'),
@@ -326,8 +332,16 @@ ALTER FOREIGN TABLE typetest1 ALTER COLUMN d TYPE timestamp(0) without time zone
 UPDATE typetest1 SET d = '1968-10-10 12:00:00' WHERE id = 1 RETURNING d;
 ROLLBACK;
 -- test if "IN" or "= ANY" expressions are pushed down correctly
+SELECT vc FROM typetest1 WHERE id IN (1, 3, 4) ORDER BY id;
+EXPLAIN (COSTS off) SELECT vc FROM typetest1 WHERE id IN (1, 3, 4) ORDER BY id;
 SELECT id FROM typetest1 WHERE vc = ANY (ARRAY['short', (SELECT 'varlena'::varchar)]) ORDER BY id;
 EXPLAIN (COSTS off) SELECT id FROM typetest1 WHERE vc = ANY (ARRAY['short', (SELECT 'varlena'::varchar)]) ORDER BY id;
+-- test NULLIF pushdown
+SELECT id FROM typetest1 WHERE nullif(id, 1) IS NULL ORDER BY id;
+EXPLAIN (COSTS off) SELECT id FROM typetest1 WHERE nullif(id, 1) IS NULL ORDER BY id;
+-- test coalesce() pushdown
+SELECT id FROM typetest1 WHERE coalesce(d, current_date) = current_date ORDER BY id;
+EXPLAIN (COSTS off) SELECT id FROM typetest1 WHERE coalesce(d, current_date) = current_date ORDER BY id;
 -- test modifications that need no foreign scan scan (bug #295)
 DELETE FROM typetest1 WHERE FALSE;
 UPDATE shorty SET c = NULL WHERE FALSE RETURNING *;
@@ -348,6 +362,8 @@ UPDATE gen SET id = 6 WHERE id = 5;
 SELECT id, c FROM gen WHERE id = 6;
 DELETE FROM gen WHERE id = 6;
 DROP FOREIGN TABLE gen;
+-- test for "ctid" in the WHERE clause (should fail)
+SELECT id FROM typetest1 WHERE ctid = '(0, 1)';
 
 /*
  * Test "strip_zeros" column option.
@@ -426,8 +442,8 @@ DELETE FROM qtest WHERE id = 5;
 
 BEGIN;
 COPY typetest1 FROM STDIN;
-666	cöpy	variation	dynamo	ünicode	Not very long	DEADF00D	9a0cf1eb-02e2-4b1f-bbe0-449fa4a99969	\\x01020304	\\xFFFF	\N	0.11111	0.43211	0.01010	2100-01-29	2050-04-01 19:30:00	12 hours	0 years
-777	fdjkl	r89809rew	^ß[]#~	\N	Das also ist des Pudels Kern.	00	fe288446-05f6-4074-9e9e-6ee41af7b377	\\x00	\\x00	FALSE	10	1002	1003	2019-05-01	2019-05-01 0:00:00	0 seconds	1 year
+666	cöpy	variation	dynamo	ünicode	Not very long	n'Clobber	DEADF00D	9a0cf1eb-02e2-4b1f-bbe0-449fa4a99969	\\x01020304	\\xFFFF	\N	0.11111	0.43211	0.01010	2100-01-29	2050-04-01 19:30:00	12 hours	0 years
+777	fdjkl	r89809rew	^ß[]#~	\N	Das also ist des Pudels Kern.	Foo	00	fe288446-05f6-4074-9e9e-6ee41af7b377	\\x00	\\x00	FALSE	10	1002	1003	2019-05-01	2019-05-01 0:00:00	0 seconds	1 year
 \.
 ROLLBACK;
 
@@ -440,8 +456,8 @@ CREATE TABLE defpart PARTITION OF party DEFAULT;
 ALTER TABLE party ATTACH PARTITION typetest1 FOR VALUES FROM (1) TO (MAXVALUE);
 BEGIN;
 COPY party FROM STDIN;
-666	cöpy	variation	dynamo	ünicode	Not very long	DEADF00D	9a0cf1eb-02e2-4b1f-bbe0-449fa4a99969	\\x01020304	\\xFFFF	\N	0.11111	0.43211	0.01010	2100-01-29	2050-04-01 19:30:00	12 hours	0 years
-777	fdjkl	r89809rew	^ß[]#~	\N	Das also ist des Pudels Kern.	00	fe288446-05f6-4074-9e9e-6ee41af7b377	\\x00	\\x00	FALSE	10	1002	1003	2019-05-01	2019-05-01 0:00:00	0 seconds	1 year
+666	cöpy	variation	dynamo	ünicode	Not very long	n'Clobber	DEADF00D	9a0cf1eb-02e2-4b1f-bbe0-449fa4a99969	\\x01020304	\\xFFFF	\N	0.11111	0.43211	0.01010	2100-01-29	2050-04-01 19:30:00	12 hours	0 years
+777	fdjkl	r89809rew	^ß[]#~	\N	Das also ist des Pudels Kern.	Foo	00	fe288446-05f6-4074-9e9e-6ee41af7b377	\\x00	\\x00	FALSE	10	1002	1003	2019-05-01	2019-05-01 0:00:00	0 seconds	1 year
 \.
 INSERT INTO party (id, lc, lr, lb)
    VALUES (12, 'very long character', '\x0001020304', '\xFFFEFDFC');
@@ -550,31 +566,6 @@ EXECUTE stmt('{varlena,nonsense}');
 DEALLOCATE stmt;
 
 
-/*
- * Test push-down of the LIMIT clause.
- */
-
--- the LIMIT clause is only pushed down with ORDER BY
-EXPLAIN (COSTS off) SELECT d FROM typetest1 LIMIT 2;
-EXPLAIN (COSTS off) SELECT d FROM typetest1 ORDER BY d LIMIT 2;
-SELECT d FROM typetest1 ORDER BY d LIMIT 2;
--- the LIMIT clause is not pushed down because the ORDER BY is not
-EXPLAIN (COSTS off) SELECT d FROM typetest1 ORDER BY lc LIMIT 2;
--- with an OFFSET clause, the offset value is added to the limit
-EXPLAIN (COSTS off) SELECT * FROM qtest ORDER BY id LIMIT 1 OFFSET 2;
-SELECT * FROM qtest ORDER BY id LIMIT 1 OFFSET 2;
--- no LIMIT push-down if there is a GROUP BY clause
-EXPLAIN (COSTS off) SELECT d, count(*) FROM typetest1 GROUP BY d LIMIT 2;
-SELECT d, count(*) FROM typetest1 GROUP BY d LIMIT 2;
--- no LIMIT push-down if there is an aggregate function
-EXPLAIN (COSTS off) SELECT 12 - count(*) FROM typetest1 LIMIT 1;
-SELECT 12 - count(*) FROM typetest1 LIMIT 1;
--- no LIMIT push-down if there is a local WHERE condition
-EXPLAIN (COSTS OFF) SELECT id FROM typetest1 WHERE vc < 'u' LIMIT 1;
-SELECT id FROM typetest1 WHERE vc < 'u' LIMIT 1;
--- no LIMIT pushdown with FOR SHARE/UPDATE
-EXPLAIN (COSTS OFF) SELECT id FROM typetest1 ORDER BY id LIMIT 1 FOR UPDATE;
-
 /* test ANALYZE */
 
 ANALYZE typetest1;
